// ****************** MUST INCLUDE LINES BELOW*****************************
#include <lib6302.h>
// Init library and set pwm freq, DAC resolution, ADC resolution, and teensy 3.6-3.2 bit rate.
// PWM frequency affects DAC resolution, higher frequency, lower resolution.
// For 8-bit PWM: Teensy 3.6-234375, Teensy 3.2-187500!!!
// Divide by 2,4,8 or 16 for 9,10,11, or 12 bits.
// WARNING 12 bit pwm is in audio range for young listeners.
#define MYSERIAL Serial
lib6302 my6302(234375/8,12,16,&Serial);
// Set up the analog-to-digital library, with a buffersize
#define sampleBufN 500
LIB6302_ADC(sampleBufN+100);  // ADC sample buffer of 100+sampleBufN
// ****************** MUST INCLUDE LINES ABOVE*******1**********************

// Constants for Users to Modify 
// ************************************************************ 
#define PLOTALL 1 // 0:meas, desired, 1:add motorCmds
#define togglePeriod 2.0 // seconds between +/- toggling of desired value.
#define highDesired 0.00 // Toggle angle in radians high
#define lowDesired -0.00 // Toggle angle in radians low
#define KPMAX 10.0 // Maximum proportional gain
#define KDMAX 0.1 // Maximum difference gain
#define KNOMMAX1 20  //  maximum offset 
#define KNOMMAX2 20 //  maximum offset 
#define sensorOffset1 1.0
#define sensorOffset2 1.0
#define mBack  4
#define plotIncrement 100
#define controllerUpdatePeriod 50 // Update period in microseconds

// ************************************************************************
// End of Constants and code for Users to Modify 
// SEE END OF FILE FOR USER-ALTERABLE CONTROL LOOP!!

// *******SETTING UP THE MONITOR DISPLAY (SLIDERS AND PLOTS). *************
// Sliders
String sliders = "S~Pause~0~1~S~Kp~0~10~S~Kd~0~5~";
#define PA 0
#define KP 1
#define KD 2

// Plots: USE SPACES BETWEEN LABELS FOR MULTIPLOTS!!
String measD = "P~D1 D2~-1~1~500~2~";
String Cmd = "P~Cmd1 Cmd2~-1~1~500~2~";
#if PLOTALL > 0
String monPlots = measD+Cmd;
#else 
String monPlots = measD;
#endif
#define PLOT_D1 0
#define PLOT_D2 1
#define PLOT_CMD1 2
#define PLOT_CMD2 3

// Configuration string with surrounding limits.
String config_message = sliders + monPlots;

// Control loop setup
// ************************************************************************

// PWM and Sensor defines.
elapsedMicros loopTimer = 0;  // Loop runs at least 10xsecond
float mDeltaT = 0.0;
float headRoom;
float Kp, Kd, nominalCmd1, nominalCmd2;

void setup() {
  MYSERIAL.begin(1000000);
  my6302.setup(config_message, togglePeriod);
  my6302.startPWM(false); 
  my6302.setupADC(16,true);
  
  loopTimer = 0;

  mDeltaT = mBack*float(controllerUpdatePeriod)*1.0e-6;
  headRoom = controllerUpdatePeriod;
  Kd = KDMAX;
  Kp = KPMAX;  
  nominalCmd1 = KNOMMAX1*(2.0*analog2param(A3) - 1.0);
  nominalCmd2 = KNOMMAX2*(2.0*analog2param(A2) - 1.0);
}

#define zeroParamThresh 0.005
#define scaleParam 1.005
float analog2param(int pinA) {
  float retval = my6302.adcGetAvg(pinA) - zeroParamThresh;
  retval = scaleParam * max(retval, 0.0);
  return(retval);
}

//********************** Main USER ALTERABLE Control Loop **********************
//**************************************************************************

int samplesSinceFlip = 0;
float lastFlip;
int loopCounter = 0;
float pastError1[mBack+1];  // Stores past errors for deriv approx
float pastCmd = 0.0;


void loop() {  

  // Initializes, gets GUI updates.
  my6302.startloop();

  float pauseVal= my6302.getSlider(PA);

  // Headroom is the faction of the period still remaining after the loop completes.
  headRoom = float(controllerUpdatePeriod - int(loopTimer));
  
  // Start exactly one period since last start.
  while (int(loopTimer) < controllerUpdatePeriod) {};
  loopTimer = 0;
  
  // Read the analog (potentiometer) values to get the controller gains.
  float meas1 = 4*my6302.adcGetAvg(A0) - sensorOffset1;
  float meas2 = 4*my6302.adcGetAvg(A2) - sensorOffset2;
  
  // Possibly toggle the desired theta.
  float flip = my6302.getFlip();
  float desired = highDesired;
  if(flip > 0) desired = lowDesired;
 
  // Compute the position error 
  float error1 = desired - (meas1-meas2);
  
  // Save backward position errors for delta calculation.
  for (int i = mBack; i > 0; i--) pastError1[i] = pastError1[i-1];
  pastError1[0] = error1;

  // Compute (error - oldError)/deltaT.
  float delta_dt1 = (error1 -pastError1[mBack])/mDeltaT;
  
  // Compute the feedback term of  the motor cmd.
  float cmd1 = Kp*error1 + Kd*delta_dt1;
  
  // Write motor command to H-bridge, returns clipped value.
  float cmdClip1 = my6302.hbridgeBipolar(nominalCmd1+cmd1, pwmA1_T, pwmA2_T);
  float cmdClip2 = my6302.hbridgeBipolar(nominalCmd2-cmd1, pwmB1_T, pwmB2_T);
  
  // Send data to monitor.
  if(loopCounter <  plotIncrement) loopCounter++;
  else {
    loopCounter = 1;
    nominalCmd1 = KNOMMAX1*(2.0*analog2param(A3) - 1.0);
    nominalCmd2 = KNOMMAX2*(2.0*analog2param(A1) - 1.0);
    
    if(pauseVal == 0) {
      // Send data to monitor.
      float loopStatus[4];
      int numPlots = 2;
      loopStatus[PLOT_D1] = meas1;
      loopStatus[PLOT_D2] = meas2;
      loopStatus[PLOT_CMD1] = cmdClip1;
      loopStatus[PLOT_CMD2] = cmdClip2;
      if(PLOTALL > 0) numPlots = 4;
      my6302.sendStatus(numPlots, loopStatus);
    } 
  }
}



  


  
